@@ -1,15 +1,12 @@
 package Rubric::Entry;
 
-use strict;
-use warnings;
-
 =head1 NAME
 
 Rubric::Entry - a single entry made by a user
 
 =head1 VERSION
 
- $Id: /my/cs/projects/rubric/trunk/lib/Rubric/Entry.pm 18100 2006-01-26T13:59:16.285684Z rjbs  $
+ $Id: Entry.pm,v 1.15 2004/12/15 17:22:29 rjbs Exp $
 
 =head1 DESCRIPTION
 
@@ -19,8 +16,6 @@ Rubric::DBI, which is a Class::DBI class.
 =cut
 
 use base qw(Rubric::DBI);
-use Encode qw(_utf8_on);
-use Rubric::Entry::Formatter;
 use Time::Piece;
 
 __PACKAGE__->table('entries');
@@ -52,12 +47,6 @@ The link attribute returns a Rubric::Link.
 
 __PACKAGE__->has_a(link => 'Rubric::Link');
 
-=head2 uri
-
-The uri attribute returns the URI of the entry's link.
-
-=cut
-
 sub uri { my ($self) = @_; return unless $self->link; $self->link->uri; }
 
 =head2 user
@@ -77,7 +66,7 @@ represent them.
 =cut
 
 __PACKAGE__->has_many(entrytags => 'Rubric::EntryTag' );
-__PACKAGE__->has_many(tags => [ 'Rubric::EntryTag' => 'tag' ]);
+__PACKAGE__->has_many(tags => [ 'Rubric::EntryTag' => tag ]);
 
 =head3 recent_tags_counted
 
@@ -89,8 +78,8 @@ pair for tags used on the week's 50 most recent entries.
 __PACKAGE__->set_sql(recent_tags_counted => <<'');
 SELECT tag, COUNT(*) as count
 FROM   entrytags
-WHERE entry IN (SELECT id FROM entries WHERE created > ? LIMIT 100)
-  AND tag NOT LIKE '@%%'
+WHERE
+	entry IN (SELECT id FROM entries WHERE created > ? LIMIT 100)
 GROUP BY tag
 ORDER BY count DESC
 LIMIT 50
@@ -114,63 +103,71 @@ inflated to Time::Piece objects.
 
 =cut
 
-__PACKAGE__->has_a(
-	$_ => 'Time::Piece',
-	deflate => 'epoch',
-	inflate => Rubric::Config->display_localtime ? sub { localtime($_[0]) }
-	                                             : sub { gmtime($_[0]) }
-) for qw(created modified);
+__PACKAGE__->has_a($_ => 'Time::Piece', deflate => 'epoch')
+	for qw(created modified);
 
-__PACKAGE__->add_trigger(before_create => \&_default_title);
+__PACKAGE__->add_trigger(before_create => \&default_title);
 
-__PACKAGE__->add_trigger(before_create => \&_create_times);
-__PACKAGE__->add_trigger(before_update => \&_update_times);
+__PACKAGE__->add_trigger(before_create => \&create_times);
+__PACKAGE__->add_trigger(before_update => \&update_times);
 
-sub _default_title {
+sub default_title {
 	my $self = shift;
 	$self->title('(default)') unless $self->{title}
 }
 
-sub _create_times {
+sub create_times {
 	my $self = shift;
-	$self->created(scalar gmtime) unless defined $self->{created};
-	$self->modified(scalar gmtime) unless defined $self->{modified};
+	$self->created(scalar gmtime) unless $self->{created};
+	$self->modified(scalar gmtime) unless $self->{modified};
 }
 
-sub _update_times {
+sub update_times {
 	my $self = shift;
 	$self->modified(scalar gmtime);
 }
 
 =head1 METHODS
 
-=head2 query(\%arg)
-
-The arguments to C<query> provide a set of abstract constraints for the query.
-These are sent to Rubric::Entry::Query, which builds an SQL query and returns
-the result of running it.  (Either a list or an Iterator is returned.)
+=head2 by_tag(\%arg)
 
+The arguments to C<by_tag> indicate the tags and users for which to search.
 (The built-in Class::DBI search method can't handle this kind of search.)
 
- user   - entries for this User
- tags   - entries with these tags (arrayref)
- link   - entries for this Link
- urimd5 - entries for the Link with this md5 sum
- has_body    - whether entries must have bodies (T, F, or undef)
- has_link    - whether entries must have a link (T, F, or undef)
- (time spec) - {created,modified}_{before,after,on}
-               limits entries by time; given as a complete or partial
-               time and date string in the form "YYYY-MM-DD HH:MM"
+ user - the user whose tags to search (can be undef)
+ tags - an arrayref of tag names
+ link - the id of the entry's link
+ has_body - whether entries must have bodies (T, F, or undef)
+ has_link - whether entries must have a link (T, F, or undef)
+
+This returns a list or Class::DBI::Iterator, depending on context.
 
 =cut
 
-sub query {
-	my $self = shift;
-	require Rubric::Entry::Query;
-	Rubric::Entry::Query->query(@_);
+sub by_tag {
+	my ($self, $arg) = @_;
+	my %wheres;
+	if ($arg->{user}) { $wheres{user} = $arg->{user} }
+	if ($arg->{link}) { $wheres{link} = $arg->{link}->id }
+	if ($arg->{tags} and my @tags = @{$arg->{tags}}) {
+		$_ = $self->db_Main->quote($_) for @tags;
+		my $ids = 
+			join ' AND ',
+			map { "id IN (SELECT entry FROM entrytags WHERE tag=$_)" }
+			@tags;
+		$wheres{''} = \$ids;
+	}
+	for (qw(body link)) {
+		if (defined $arg->{"has_$_"}) {
+			$wheres{$_} = $arg->{"has_$_"} ? \'IS NOT NULL' : \'IS NULL';
+		}
+	}
+	%wheres
+		? $self->search_where(\%wheres, { order_by => "created DESC" })
+		: $self->retrieve_all;
 }
 
-=head2 set_new_tags(\%tags)
+=head2 set_new_tags(\@tags)
 
 This method replaces all entry's current tags with the new set of tags.
 
@@ -181,72 +178,22 @@ sub set_new_tags {
 	$self->entrytags->delete_all;
 	$self->update;
 
-  while (my ($tag, $value) = each %$tags) {
-    $self->add_to_tags({ tag => $tag, tag_value => $value });
-  }
+	$self->add_to_tags({ tag => $_ }) for @$tags;
 }
 
 =head2 tags_from_string($taglist)
 
 This (class) method takes a string of tags, delimited by whitespace, and
-returns an array of the tags, dropping invalid tags.
+returns a reference to an array of the tags, dropping invalid tags.
 
 Valid tags (shouldn't this be documented somewhere else instead?) may contain
-letters, numbers, underscores, colons, dots, and asterisks.  Hyphens me be
-used, but not as the first character.
+letters, numbers, underscores, colons, dots, and asterisks.
 
 =cut
 
 sub tags_from_string {
-	my ($class, $tagstring) = @_;
-	my %seen;
-
-	_utf8_on($tagstring);
-
-  return {} unless $tagstring;
-
-  # remove leading and trailing spaces
-  $tagstring =~ s/\A\s*//;
-  # I originally used this line, but trimming trailing space is not needed, so
-  # I reverted to the simpler code, above: $tagstring =~ s/\A\s*(.+?)\s*\z/$1/;
-
-	my %tags = $tagstring
-           ? map { (index($_, ':') > 0) ? split(/:/, $_, 2) : ($_ => undef) }
-	               split /(?:\+|\s)+/, $tagstring
-	         : ();
-
-	die "invalid characters in tagstring" 
-		if grep { defined $_ and $_ !~ /\A[@\w\d:.*][-\w\d:.*]*\Z/ } keys %tags;
-
-	die "invalid characters in tagstring" 
-		if grep { defined $_ and $_ !~ /\A[-\w\d:.*]*\Z/ } values %tags;
-
-	return \%tags;
-}
-
-=head2 C< body_as >
-
-  my $formatted_body = $entry->body_as("html");
-
-This method returns the body of the entry, formatted into the given format.  If
-the entry cannot be rendered into the given format, an exception is thrown.
-
-=cut
-
-sub body_as {
-  my ($self, $format) = @_;
-
-  my $markup;
-  my ($tag)
-    = Rubric::EntryTag->search({ entry => $self->id, tag => '@markup' });
-
-  $markup = ($tag and $tag->tag_value) ? $tag->tag_value : '_default';
-
-  Rubric::Entry::Formatter->format({
-    text   => $self->body,
-    markup => $markup,
-    format => $format
-  });
+	my ($class, $taglist) = @_;
+	[ grep /^[\w\d:.*]+$/, split /\s+/, $taglist ];
 }
 
 ## return retrieve_all'd objects in recent-to-older order
